<?php

/**
 * @file
 * Dominion hooks file.
 */

function hook_dominion_load(&$domain) {
  // Add a variable to the $domain array.
  $domain['myvar'] = 'mydominionvar';
  // Remove the site_grant flag, removing access to 'all affiliates.'
  $domain['site_grant'] = FALSE;
}

/**
 * Notify other modules that we have created a new domain.
 *
 * @param $domain
 *  The domain record taken from {domain}, as an array.
 * @param $form_values
 *   The form values processed by the form.  Note that these are not editable
 *   since module_invoke_all() cannot pass by reference. They are passed in
 *   case some module needs to check the original form input.
 *
 * @ingroup dominion_hooks
 */
function hook_dominion_insert($domain, $form_values = array()) {
  db_insert('mytable')
  ->fields(array(
      'subdomain' => $domain['subdomain'],
      'sitename' => $domain['sitename'],
  ))
  ->execute();
}

/**
 * Notify other modules that we have updated a domain.
 *
 * @param $domain
 *  The domain record taken from {domain}, as an array.
 * @param $form_values
 *   The form values processed by the form.  Note that these are not editable
 *   since module_invoke_all() cannot pass by reference. They are passed in
 *   case some module needs to check the original form input.
 *
 * @ingroup domain_hooks
 */
function hook_dominion_update($domain, $form_values = array()) {
  db_update('mytable')
  ->fields(array(
    'subdomain' => $domain['subdomain'],
    'sitename' => $domain['sitename'],
  ))
  ->condition('domain_id', $domain['domain_id'])
  ->execute();
}

/**
 * Notify other modules that we have deleted a domain.
 *
 * @param $domain
 *  The domain record taken from {domain}, as an array.
 * @param $form_values
 *   The form values processed by the form.  Note that these are not editable
 *   since module_invoke_all() cannot pass by reference. They are passed in
 *   case some module needs to check the original form input.
 *
 * @ingroup domain_hooks
 */
function hook_dominion_delete($domain, $form_values = array()) {
  db_delete('mytable')
  ->condition('subdomain', $domain['subdomain']);
}

/**
 * Hook for actions taken on subsites handled by dominion.
 *
 * @param string $op
 * @param int $domain_id
 * @param mixed $a3
 */
/*
 function hook_dominion($op, $domain_id, &$a3 = NULL) {
 switch ($op) {
 case 'insert':
 // $a3 is an associative array with all values from dominion_form.
 break;
 case 'update':
 // $a3 is an associative array with all values from dominion_form.
 break;
 case 'delete':
 // $a3 is an object from dominion_get_info().
 break;
 case 'load':
 // $a3 is an object from dominion_get_info(), you may alter this object here.
 break;
 }
 }*/

/**
 * Hook for defining functions available in dominion subsites.
 *
 * @return array
 */
function hook_dominion_functions() {
  return array(
    'forum' => t('Forum'),
    'blog' => t('Blog'),
  );
}
