<?php
/**
 * @file
 * Admin functions for dominion_menu.
 */


/**
 * Admin settings form.
 */
function dominion_menu_admin_settings($form, &$form_state) {
  $form = array();

  $form['rebuild'] = array(
    '#type' => 'fieldset',
    '#title' => t('Rebuild menu\'s'),
    '#description' => t('With this button you can rebuild the menu\'s for all subsites. This will rename all menu\'s and create menu\'s which do not exists already. Links within the menu\'s are kept as-is.'),
  );
  $form['rebuild']['rebuild'] = array(
    '#type' => 'submit',
    '#value' => t('Rebuild menu\'s'),
  );

  $form[DOMINION_MENU_VAR_CREATE_PRIMARY_MENU] = array(
    '#type' => 'checkbox',
    '#title' => t('Create primary menu'),
    '#default_value' => variable_get(DOMINION_MENU_VAR_CREATE_PRIMARY_MENU, DOMINION_MENU_VAR_CREATE_PRIMARY_MENU_DEFAULT),
  );

  $form[DOMINION_MENU_VAR_DEFAULT_PRIMARY_MENU] = array(
    '#type' => 'textarea',
    '#title' => t('Default primary menu'),
    '#default_value' => variable_get(DOMINION_MENU_VAR_DEFAULT_PRIMARY_MENU, DOMINION_MENU_VAR_DEFAULT_PRIMARY_MENU_DEFAULT),
    '#description' => t('Default menu to create for new subsites. One item per line, each having a path, title and optional a required function name separated by pipes.'),
    '#wysiwyg' => FALSE,
  );

  $form[DOMINION_MENU_VAR_CREATE_SECONDARY_MENU] = array(
    '#type' => 'checkbox',
    '#title' => t('Create secondary menu'),
    '#default_value' => variable_get(DOMINION_MENU_VAR_CREATE_SECONDARY_MENU, DOMINION_MENU_VAR_CREATE_SECONDARY_MENU_DEFAULT),
  );

  $form[DOMINION_MENU_VAR_DEFAULT_SECONDARY_MENU] = array(
    '#type' => 'textarea',
    '#title' => t('Default secondary menu'),
    '#default_value' => variable_get(DOMINION_MENU_VAR_DEFAULT_SECONDARY_MENU, DOMINION_MENU_VAR_DEFAULT_SECONDARY_MENU_DEFAULT),
    '#description' => t('Default menu to create for new subsites. One item per line, each having a path, title and optional a required function name separated by pipes.'),
    '#wysiwyg' => FALSE,
  );

  $form[DOMINION_MENU_VAR_PRIMARY_MENU_TITLE] = array(
    '#type' => 'textfield',
    '#title' => t('Primary menu title'),
    '#default_value' => variable_get(DOMINION_MENU_VAR_PRIMARY_MENU_TITLE, DOMINION_MENU_VAR_PRIMARY_MENU_TITLE_DEFAULT),
    '#required' => TRUE,
  );

  $form[DOMINION_MENU_VAR_SECONDARY_MENU_TITLE] = array(
    '#type' => 'textfield',
    '#title' => t('Secondary menu title'),
    '#default_value' => variable_get(DOMINION_MENU_VAR_SECONDARY_MENU_TITLE, DOMINION_MENU_VAR_SECONDARY_MENU_TITLE_DEFAULT),
    '#required' => TRUE,
  );

  $form['#submit'][] = 'dominion_menu_admin_settings_submit';

  return system_settings_form($form);
}

function dominion_menu_admin_settings_submit($form, &$form_state) {

  if (strstr($form_state['clicked_button']['#id'], 'edit-rebuild')) {
    module_load_include('inc', 'dominion_menu');
    $renamed = 0;
    $created = 0;

    // SELECT domain_id FROM domain.
    $query = db_select('domain', 'do');
    $query->fields('do', array('domain_id'));
    $res = $query->execute();

    while ($rec = $res->fetchAssoc()) {
      if ($subsite = dominion_get_info($rec['domain_id'])) {
        // Check primary menu.
        $create = variable_get(DOMINION_MENU_VAR_CREATE_PRIMARY_MENU, DOMINION_MENU_VAR_CREATE_PRIMARY_MENU_DEFAULT);
        if ($menu = _dominion_menu_exists($rec['domain_id'], TRUE)) {
          $template = variable_get(DOMINION_MENU_VAR_PRIMARY_MENU_TITLE, DOMINION_MENU_VAR_PRIMARY_MENU_TITLE_DEFAULT);
          $title = drupal_substr(t($template, array('@name' => $subsite->sitename)), 0, MENU_MAX_MENU_NAME_LENGTH_UI);

          //'UPDATE {menu_custom} SET title = $title WHERE menu_name = $menu;
          $query = db_update('menu_custom');
          $query->fields(array('title' => $title));
          $query->condition('menu_name', $menu);
          $query->execute();

          ++$renamed;
        }
        elseif ($create) {
          _dominion_menu_create($rec['domain_id'], $subsite->available_functions, TRUE);
          ++$created;
        }
        // Check secondary menu.
        $create = variable_get(DOMINION_MENU_VAR_CREATE_SECONDARY_MENU, DOMINION_MENU_VAR_CREATE_SECONDARY_MENU_DEFAULT);


        if ($menu = _dominion_menu_exists($rec['domain_id'], FALSE)) {
          dpm( $rec['domain_id'] . '-' . $menu );
          $template = variable_get(DOMINION_MENU_VAR_SECONDARY_MENU_TITLE, DOMINION_MENU_VAR_SECONDARY_MENU_TITLE_DEFAULT);
          $title = drupal_substr(t($template, array('@name' => $subsite->sitename)), 0, MENU_MAX_MENU_NAME_LENGTH_UI);

          // UPDATE {menu_custom} SET title = $title WHERE menu_name = $menu.
          $query = db_update('menu_custom');
          $query->fields(array('title' => $title));
          $query->condition('menu_name', $menu);
          $query->execute();

          ++$renamed;
        }
        elseif ($create) {
          _dominion_menu_create($rec['domain_id'], $subsite->available_functions, FALSE);
          ++$created;
        }
      }
    }
    drupal_set_message(t(
      '!renamed menu\'s were renamed and !created menu\'s were created.',
      array(
        '!renamed' => $renamed,
        '!created' => $created,
      )
    ));
  }
}
