<?php
/**
 * @file
 * Inclusion file of functions for dominion_menu.
 */

/**
 * Get the (machine readable) name for a subsite menu.
 *
 * @param int $domain_id
 * @param bool $primary
 * @return string
 */
function _dominion_menu_get_name($domain_id, $primary) {
  $template = $primary ? DOMINION_MENU_PRIMARY_NAME : DOMINION_MENU_SECONDARY_NAME;
  return str_replace('%', $domain_id, $template);
}

/**
 * Create a subsite menu.
 *
 * @param int $domain_id
 * @param array $functions
 * @param bool $primary
 */
function _dominion_menu_create($domain_id, $functions, $primary) {
  $menu = _dominion_menu_get_name($domain_id, $primary);

  if ($primary) {
    $configuration = variable_get(DOMINION_MENU_VAR_DEFAULT_PRIMARY_MENU, DOMINION_MENU_VAR_DEFAULT_PRIMARY_MENU_DEFAULT);
  }
  else {
    $configuration = variable_get(DOMINION_MENU_VAR_DEFAULT_SECONDARY_MENU, DOMINION_MENU_VAR_DEFAULT_SECONDARY_MENU_DEFAULT);
  }

  // Add a menu using the menu_edit_menu form
  if ($primary) {
    $template = variable_get(DOMINION_MENU_VAR_PRIMARY_MENU_TITLE, DOMINION_MENU_VAR_PRIMARY_MENU_TITLE_DEFAULT);
  }
  else {
    $template = variable_get(DOMINION_MENU_VAR_SECONDARY_MENU_TITLE, DOMINION_MENU_VAR_SECONDARY_MENU_TITLE_DEFAULT);
  }
  if ($subsite = dominion_get_info($domain_id)) {
    $title = drupal_substr(t($template, array('@name' => $subsite->sitename)), 0, MENU_MAX_MENU_NAME_LENGTH_UI);
  }

  $menu_edit_menu_form_state = array(
    'values' => array(
      'menu_name' => $menu,
      'title' => drupal_substr(t($template, array('@name' => $title)), 0, MENU_MAX_MENU_NAME_LENGTH_UI),
      'description' => '',
  )
  );
  module_load_include('inc', 'menu', 'menu.admin');
  // drupal_execute('menu_edit_menu', $menu_edit_menu_form_state, 'add');
  drupal_form_submit('menu_edit_menu', $menu_edit_menu_form_state, 'add');

  $variable = $primary ? 'menu_primary_links_source' : 'menu_secondary_links_source';
  domain_conf_variable_set($domain_id, $variable, "menu-$menu");

  // Add menu items.
  $weight = 0;
  $items = array_filter(array_map('trim', explode("\n", $configuration)));
  foreach ($items as $item) {
    $item = array_filter(array_map('trim', explode('|', $item)));
    if (isset($item[2]) && empty($functions[$item[2]])) {
      // This subsite does not have the required function enabled.
      continue;
    }
    ++$weight;
    $item = array(
      'link_path' => $item[0],
      'link_title' => $item[1],
      'weight' => $weight,
      'menu_name' => "menu-$menu",
    );
    menu_link_save($item);
  }
}

/**
 * Delete a subsite menu.
 *
 * @param int $domain_id
 * @param bool $primary
 */
function _dominion_menu_delete($domain_id, $primary) {
  if (!_dominion_menu_exists($domain_id, $primary)) {
    return;
  }
  // Delete menu using the menu_delete_menu_confirm form.
  $variable = $primary ? 'menu_primary_links_source' : 'menu_secondary_links_source';
  $menu = domain_conf_variable_get($domain_id, $variable);
  $sql = 'SELECT * FROM {menu_custom} WHERE menu_name = \'%s\'';
  $res = db_query($sql, $menu);
  if ($menu = db_fetch_array($res)) {
    $menu_delete_menu_confirm_form_state = array('#values' => array());
    module_load_include('inc', 'menu', 'menu.admin');
    menu_delete_menu_confirm_submit(array('#menu' => $menu), $menu_delete_menu_confirm_form_state);
  }
}

/**
 * Check if a dominion menu exists.
 *
 * @param int $domain_id
 * @param bool $primary
 * @return mixed
 *   Menu name or false when menu does not exists.
 */
function _dominion_menu_exists($domain_id, $primary) {
  $variable = ($primary) ? 'menu_primary_links_source' : 'menu_secondary_links_source';

  $value = domain_conf_variable_get($domain_id, $variable);
  $default_value = $primary ? 'primary-links' : 'secondary-links';
  if (empty($value) || $value == $default_value) {
    return FALSE;
  }
  //'SELECT * FROM {menu_custom} WHERE menu_name = \'%s\' LIMIT 1';
  $query = db_select('menu_custom', 'mc');
  $query->fields('mc', array('menu_name'));
  $query->condition('mc.menu_name', $value);
  $query->range(0, 1);
  $res = $query->execute();
  $result = $res->fetchAssoc();
  //dpm($result);
  if ($res->fetchAssoc()) {
    //dpm($value);
    return $value;
  }
  return FALSE;
}
